@@ -5,15 +5,18 @@ TODO:
  - better support for relations
  - better support for constraints (mainly postgresql?)
  - support for table spaces with postgresql
- 
+ - when a table is not managed (meta.managed==False) then only do a one-way
+   sqldiff ? show differences from db->table but not the other way around since
+   it's not managed.
+
 KNOWN ISSUES:
  - MySQL has by far the most problems with introspection. Please be
-   carefull when using MySQL with sqldiff. 
+   carefull when using MySQL with sqldiff.
    - Booleans are reported back as Integers, so there's know way to know if
      there was a real change.
-   - Varchar sizes are reported back without unicode support so there size
-     may change in comparison to the real length of the varchar.   
-   - Some of the 'fixes' to counter these problems might create false 
+   - Varchar sizes are reported back without unicode support so their size
+     may change in comparison to the real length of the varchar.
+   - Some of the 'fixes' to counter these problems might create false
      positives or false negatives.
 """
 
@@ -27,6 +30,7 @@ from optparse import make_option
 
 ORDERING_FIELD = IntegerField('_order', null=True)
 
+
 def flatten(l, ltypes=(list, tuple)):
     ltype = type(l)
     l = list(l)
@@ -42,11 +46,19 @@ def flatten(l, ltypes=(list, tuple)):
         i += 1
     return ltype(l)
 
+
+def all_local_fields(meta):
+    all_fields = meta.local_fields[:]
+    for parent in meta.parents:
+        all_fields.extend(all_local_fields(parent._meta))
+    return all_fields
+
+
 class SQLDiff(object):
-    DATA_TYPES_REVERSE_OVERRIDE = {
-    }
-    
+    DATA_TYPES_REVERSE_OVERRIDE = {}
+
     DIFF_TYPES = [
+        'error',
         'comment',
         'table-missing-in-db',
         'field-missing-in-db',
@@ -59,16 +71,17 @@ class SQLDiff(object):
         'field-parameter-differ',
     ]
     DIFF_TEXTS = {
+        'error': 'error: %(0)s',
         'comment': 'comment: %(0)s',
         'table-missing-in-db': "table '%(0)s' missing in database",
-        'field-missing-in-db' : "field '%(1)s' defined in model but missing in database",
-        'field-missing-in-model' : "field '%(1)s' defined in database but missing in model",
-        'index-missing-in-db' : "field '%(1)s' INDEX defined in model but missing in database",
-        'index-missing-in-model' : "field '%(1)s' INDEX defined in database schema but missing in model",
-        'unique-missing-in-db' : "field '%(1)s' UNIQUE defined in model but missing in database",
-        'unique-missing-in-model' : "field '%(1)s' UNIQUE defined in database schema but missing in model",
-        'field-type-differ' : "field '%(1)s' not of same type: db='%(3)s', model='%(2)s'",
-        'field-parameter-differ' : "field '%(1)s' parameters differ: db='%(3)s', model='%(2)s'",
+        'field-missing-in-db': "field '%(1)s' defined in model but missing in database",
+        'field-missing-in-model': "field '%(1)s' defined in database but missing in model",
+        'index-missing-in-db': "field '%(1)s' INDEX defined in model but missing in database",
+        'index-missing-in-model': "field '%(1)s' INDEX defined in database schema but missing in model",
+        'unique-missing-in-db': "field '%(1)s' UNIQUE defined in model but missing in database",
+        'unique-missing-in-model': "field '%(1)s' UNIQUE defined in database schema but missing in model",
+        'field-type-differ': "field '%(1)s' not of same type: db='%(3)s', model='%(2)s'",
+        'field-parameter-differ': "field '%(1)s' parameters differ: db='%(3)s', model='%(2)s'",
     }
 
     SQL_FIELD_MISSING_IN_DB = lambda self, style, qn, args: "%s %s\n\t%s %s %s;" % (style.SQL_KEYWORD('ALTER TABLE'), style.SQL_TABLE(qn(args[0])), style.SQL_KEYWORD('ADD'), style.SQL_FIELD(qn(args[1])), style.SQL_COLTYPE(args[2]))
@@ -79,8 +92,9 @@ class SQLDiff(object):
     SQL_UNIQUE_MISSING_IN_DB = lambda self, style, qn, args: "%s %s\n\t%s %s (%s);" % (style.SQL_KEYWORD('ALTER TABLE'), style.SQL_TABLE(qn(args[0])), style.SQL_KEYWORD('ADD'), style.SQL_KEYWORD('UNIQUE'), style.SQL_FIELD(qn(args[1])))
     # FIXME: need to lookup unique constraint name instead of appending _key to table + fieldname
     SQL_UNIQUE_MISSING_IN_MODEL = lambda self, style, qn, args: "%s %s\n\t%s %s %s;" % (style.SQL_KEYWORD('ALTER TABLE'), style.SQL_TABLE(qn(args[0])), style.SQL_KEYWORD('DROP'), style.SQL_KEYWORD('CONSTRAINT'), style.SQL_TABLE(qn("%s_key" % ('_'.join(args[:2])))))
-    SQL_FIELD_TYPE_DIFFER = lambda self, style, qn, args:  "%s %s\n\t%s %s %s;" % (style.SQL_KEYWORD('ALTER TABLE'), style.SQL_TABLE(qn(args[0])), style.SQL_KEYWORD("MODIFY"), style.SQL_FIELD(qn(args[1])), style.SQL_COLTYPE(args[2]))
-    SQL_FIELD_PARAMETER_DIFFER = lambda self, style, qn, args:  "%s %s\n\t%s %s %s;" % (style.SQL_KEYWORD('ALTER TABLE'), style.SQL_TABLE(qn(args[0])), style.SQL_KEYWORD("MODIFY"), style.SQL_FIELD(qn(args[1])), style.SQL_COLTYPE(args[2]))
+    SQL_FIELD_TYPE_DIFFER = lambda self, style, qn, args: "%s %s\n\t%s %s %s;" % (style.SQL_KEYWORD('ALTER TABLE'), style.SQL_TABLE(qn(args[0])), style.SQL_KEYWORD("MODIFY"), style.SQL_FIELD(qn(args[1])), style.SQL_COLTYPE(args[2]))
+    SQL_FIELD_PARAMETER_DIFFER = lambda self, style, qn, args: "%s %s\n\t%s %s %s;" % (style.SQL_KEYWORD('ALTER TABLE'), style.SQL_TABLE(qn(args[0])), style.SQL_KEYWORD("MODIFY"), style.SQL_FIELD(qn(args[1])), style.SQL_COLTYPE(args[2]))
+    SQL_ERROR = lambda self, style, qn, args: style.NOTICE('-- Error: %s' % style.ERROR(args[0]))
     SQL_COMMENT = lambda self, style, qn, args: style.NOTICE('-- Comment: %s' % style.SQL_TABLE(args[0]))
     SQL_TABLE_MISSING_IN_DB = lambda self, style, qn, args: style.NOTICE('-- Table missing: %s' % args[0])
 
@@ -100,8 +114,9 @@ class SQLDiff(object):
         self.db_tables = self.introspection.get_table_list(self.cursor)
         self.differences = []
         self.unknown_db_fields = {}
-        
+
         self.DIFF_SQL = {
+            'error': self.SQL_ERROR,
             'comment': self.SQL_COMMENT,
             'table-missing-in-db': self.SQL_TABLE_MISSING_IN_DB,
             'field-missing-in-db': self.SQL_FIELD_MISSING_IN_DB,
@@ -114,10 +129,9 @@ class SQLDiff(object):
             'field-parameter-differ': self.SQL_FIELD_PARAMETER_DIFFER,
         }
 
-
     def add_app_model_marker(self, app_label, model_name):
         self.differences.append((app_label, model_name, []))
-        
+
     def add_difference(self, diff_type, *args):
         assert diff_type in self.DIFF_TYPES, 'Unknown difference type'
         self.differences[-1][-1].append((diff_type, args))
@@ -134,13 +148,13 @@ class SQLDiff(object):
                 django_tables = _sql.django_table_list(only_existing=only_existing)
         return django_tables
 
-    def sql_to_dict(self, query,param):
+    def sql_to_dict(self, query, param):
         """ sql_to_dict(query, param) -> list of dicts
 
         code from snippet at http://www.djangosnippets.org/snippets/1383/
         """
         cursor = connection.cursor()
-        cursor.execute(query,param)
+        cursor.execute(query, param)
         fieldnames = [name[0] for name in cursor.description]
         result = []
         for row in cursor.fetchall():
@@ -151,7 +165,7 @@ class SQLDiff(object):
         return result
 
     def get_field_model_type(self, field):
-        return field.db_type()
+        return field.db_type(connection=connection)
 
     def get_field_db_type(self, description, field=None, table_name=None):
         from django.db import models
@@ -174,7 +188,7 @@ class SQLDiff(object):
                     self.unknown_db_fields[key] = 1
                     self.add_difference('comment', "Unknown database type for field '%s' (%s)" % (description[0], type_code))
                 return None
-        
+
         kwargs = {}
         if isinstance(reverse_type, tuple):
             kwargs.update(reverse_type[1])
@@ -185,23 +199,31 @@ class SQLDiff(object):
 
         if reverse_type == "DecimalField":
             kwargs['max_digits'] = description[4]
-            kwargs['decimal_places'] = description[5]
+            kwargs['decimal_places'] = description[5] and abs(description[5]) or description[5]
 
         if description[6]:
             kwargs['blank'] = True
             if not reverse_type in ('TextField', 'CharField'):
                 kwargs['null'] = True
 
-        field_db_type = getattr(models, reverse_type)(**kwargs).db_type()
+        if '.' in reverse_type:
+            from django.utils import importlib
+            # TODO: when was importlib added to django.utils ? and do we
+            # need to add backwards compatibility code ?
+            module_path, package_name = reverse_type.rsplit('.', 1)
+            module = importlib.import_module(module_path)
+            field_db_type = getattr(module, package_name)(**kwargs).db_type(connection=connection)
+        else:
+            field_db_type = getattr(models, reverse_type)(**kwargs).db_type(connection=connection)
         return field_db_type
 
     def strip_parameters(self, field_type):
-        if field_type:
+        if field_type and field_type != 'double precision':
             return field_type.split(" ")[0].split("(")[0]
         return field_type
 
     def find_unique_missing_in_db(self, meta, table_indexes, table_name):
-        for field in meta.fields:
+        for field in all_local_fields(meta):
             if field.unique:
                 attname = field.db_column or field.attname
                 if attname in table_indexes and table_indexes[attname]['unique']:
@@ -211,28 +233,33 @@ class SQLDiff(object):
     def find_unique_missing_in_model(self, meta, table_indexes, table_name):
         # TODO: Postgresql does not list unique_togethers in table_indexes
         #       MySQL does
-        fields = dict([(field.db_column or field.name, field.unique) for field in meta.fields])
+        fields = dict([(field.db_column or field.name, field.unique) for field in all_local_fields(meta)])
         for att_name, att_opts in table_indexes.iteritems():
             if att_opts['unique'] and att_name in fields and not fields[att_name]:
-                if att_name in flatten(meta.unique_together): continue
+                if att_name in flatten(meta.unique_together):
+                    continue
                 self.add_difference('unique-missing-in-model', table_name, att_name)
 
     def find_index_missing_in_db(self, meta, table_indexes, table_name):
-        for field in meta.fields:
+        for field in all_local_fields(meta):
             if field.db_index:
                 attname = field.db_column or field.attname
                 if not attname in table_indexes:
                     self.add_difference('index-missing-in-db', table_name, attname)
 
     def find_index_missing_in_model(self, meta, table_indexes, table_name):
-        fields = dict([(field.name, field) for field in meta.fields])
+        fields = dict([(field.name, field) for field in all_local_fields(meta)])
         for att_name, att_opts in table_indexes.iteritems():
             if att_name in fields:
                 field = fields[att_name]
-                if field.db_index: continue
-                if att_opts['primary_key'] and field.primary_key: continue
-                if att_opts['unique'] and field.unique: continue
-                if att_opts['unique'] and att_name in flatten(meta.unique_together): continue
+                if field.db_index:
+                    continue
+                if att_opts['primary_key'] and field.primary_key:
+                    continue
+                if att_opts['unique'] and field.unique:
+                    continue
+                if att_opts['unique'] and att_name in flatten(meta.unique_together):
+                    continue
                 self.add_difference('index-missing-in-model', table_name, att_name)
 
     def find_field_missing_in_model(self, fieldmap, table_description, table_name):
@@ -244,12 +271,14 @@ class SQLDiff(object):
         db_fields = [row[0] for row in table_description]
         for field_name, field in fieldmap.iteritems():
             if field_name not in db_fields:
-                self.add_difference('field-missing-in-db', table_name, field_name, field.db_type())
+                self.add_difference('field-missing-in-db', table_name, field_name, 
+                                                           field.db_type(connection=connection))
 
     def find_field_type_differ(self, meta, table_description, table_name, func=None):
         db_fields = dict([(row[0], row) for row in table_description])
-        for field in meta.fields:
-            if field.name not in db_fields: continue
+        for field in all_local_fields(meta):
+            if field.name not in db_fields:
+                continue
             description = db_fields[field.name]
 
             model_type = self.strip_parameters(self.get_field_model_type(field))
@@ -259,28 +288,29 @@ class SQLDiff(object):
             if func:
                 model_type, db_type = func(field, description, model_type, db_type)
 
-            if not model_type==db_type:
+            if not model_type == db_type:
                 self.add_difference('field-type-differ', table_name, field.name, model_type, db_type)
 
     def find_field_parameter_differ(self, meta, table_description, table_name, func=None):
         db_fields = dict([(row[0], row) for row in table_description])
-        for field in meta.fields:
-            if field.name not in db_fields: continue
+        for field in all_local_fields(meta):
+            if field.name not in db_fields:
+                continue
             description = db_fields[field.name]
 
             model_type = self.get_field_model_type(field)
             db_type = self.get_field_db_type(description, field, table_name)
 
-            if not self.strip_parameters(model_type)==self.strip_parameters(db_type):
+            if not self.strip_parameters(model_type) == self.strip_parameters(db_type):
                 continue
-            
+
             # use callback function if defined
             if func:
                 model_type, db_type = func(field, description, model_type, db_type)
 
-            if not model_type==db_type:
+            if not model_type == db_type:
                 self.add_difference('field-parameter-differ', table_name, field.name, model_type, db_type)
-    
+
     @transaction.commit_manually
     def find_differences(self):
         cur_app_label = None
@@ -289,7 +319,7 @@ class SQLDiff(object):
             table_name = meta.db_table
             app_label = meta.app_label
 
-            if cur_app_label!=app_label:
+            if cur_app_label != app_label:
                 # Marker indicating start of difference scan for this table_name
                 self.add_app_model_marker(app_label, app_model.__name__)
 
@@ -298,10 +328,10 @@ class SQLDiff(object):
                 # Table is missing from database
                 self.add_difference('table-missing-in-db', table_name)
                 continue
-            
+
             table_indexes = self.introspection.get_indexes(self.cursor, table_name)
-            fieldmap = dict([(field.db_column or field.get_attname(), field) for field in meta.fields])
-            
+            fieldmap = dict([(field.db_column or field.get_attname(), field) for field in all_local_fields(meta)])
+
             # add ordering field if model uses order_with_respect_to
             if meta.order_with_respect_to:
                 fieldmap['_order'] = ORDERING_FIELD
@@ -309,10 +339,11 @@ class SQLDiff(object):
             try:
                 table_description = self.introspection.get_table_description(self.cursor, table_name)
             except Exception, e:
-                model_diffs.append((app_model.__name__, [str(e).strip()]))
-                transaction.rollback() # reset transaction
+                self.add_difference('error', 'unable to introspect table: %s' % str(e).strip())
+                transaction.rollback()  # reset transaction
                 continue
-            
+            else:
+                transaction.commit()
             # Fields which are defined in database but not in model
             # 1) find: 'unique-missing-in-model'
             self.find_unique_missing_in_model(meta, table_indexes, table_name)
@@ -345,7 +376,8 @@ class SQLDiff(object):
     def print_diff_text(self, style):
         cur_app_label = None
         for app_label, model_name, diffs in self.differences:
-            if not diffs: continue
+            if not diffs:
+                continue
             if not self.dense and cur_app_label != app_label:
                 print style.NOTICE("+ Application:"), style.SQL_TABLE(app_label)
                 cur_app_label = app_label
@@ -354,34 +386,42 @@ class SQLDiff(object):
             for diff in diffs:
                 diff_type, diff_args = diff
                 text = self.DIFF_TEXTS[diff_type] % dict((str(i), style.SQL_TABLE(e)) for i, e in enumerate(diff_args))
-                text = "'".join(i%2==0 and style.ERROR_OUTPUT(e) or e for i, e in enumerate(text.split("'")))
+                text = "'".join(i % 2 == 0 and style.ERROR(e) or e for i, e in enumerate(text.split("'")))
                 if not self.dense:
                     print style.NOTICE("|--+"), text
                 else:
-                    print style.NOTICE("App"), style.SQL_TABLE(app_name), style.NOTICE('Model'), style.SQL_TABLE(model_name), text
+                    print style.NOTICE("App"), style.SQL_TABLE(app_label), style.NOTICE('Model'), style.SQL_TABLE(model_name), text
 
     def print_diff_sql(self, style):
         cur_app_label = None
         qn = connection.ops.quote_name
-        print style.SQL_KEYWORD("BEGIN;")
-        for app_label, model_name, diffs in self.differences:
-            if not diffs: continue
-            if not self.dense and cur_app_label != app_label:
-                print style.NOTICE("-- Application: %s" % style.SQL_TABLE(app_label))
-                cur_app_label = app_label
+        has_differences = max([len(diffs) for app_label, model_name, diffs in self.differences])
+        if not has_differences:
             if not self.dense:
-                print style.NOTICE("-- Model: %s" % style.SQL_TABLE(model_name))
-            for diff in diffs:
-                diff_type, diff_args = diff
-                text = self.DIFF_SQL[diff_type](style, qn, diff_args)
-                if self.dense:
-                    text = text.replace("\n\t", " ")
-                print text
-        print style.SQL_KEYWORD("COMMIT;")
-        
+                print style.SQL_KEYWORD("-- No differences")
+        else:
+            print style.SQL_KEYWORD("BEGIN;")
+            for app_label, model_name, diffs in self.differences:
+                if not diffs:
+                    continue
+                if not self.dense and cur_app_label != app_label:
+                    print style.NOTICE("-- Application: %s" % style.SQL_TABLE(app_label))
+                    cur_app_label = app_label
+                if not self.dense:
+                    print style.NOTICE("-- Model: %s" % style.SQL_TABLE(model_name))
+                for diff in diffs:
+                    diff_type, diff_args = diff
+                    text = self.DIFF_SQL[diff_type](style, qn, diff_args)
+                    if self.dense:
+                        text = text.replace("\n\t", " ")
+                    print text
+            print style.SQL_KEYWORD("COMMIT;")
+
+
 class GenericSQLDiff(SQLDiff):
     pass
 
+
 class MySQLDiff(SQLDiff):
     # All the MySQL hacks together create something of a problem
     # Fixing one bug in MySQL creates another issue. So just keep in mind
@@ -392,39 +432,40 @@ class MySQLDiff(SQLDiff):
         # if i remember correctly it had something todo with unicode strings
         # TODO: Fix this is a more meaningful and better understood manner
         description = list(description)
-        if description[1] not in [FIELD_TYPE.TINY, FIELD_TYPE.SHORT]: # exclude tinyints from conversion.
-            description[3] = description[3]/3
-            description[4] = description[4]/3
+        if description[1] not in [FIELD_TYPE.TINY, FIELD_TYPE.SHORT]:  # exclude tinyints from conversion.
+            description[3] = description[3] / 3
+            description[4] = description[4] / 3
         db_type = super(MySQLDiff, self).get_field_db_type(description)
         if not db_type:
             return
         if field:
-            if field.primary_key and db_type=='integer':
+            if field.primary_key and (db_type == 'integer' or db_type == 'bigint'):
                 db_type += ' AUTO_INCREMENT'
             # MySQL isn't really sure about char's and varchar's like sqlite
             field_type = self.get_field_model_type(field)
             # Fix char/varchar inconsistencies
-            if self.strip_parameters(field_type)=='char' and self.strip_parameters(db_type)=='varchar':
+            if self.strip_parameters(field_type) == 'char' and self.strip_parameters(db_type) == 'varchar':
                 db_type = db_type.lstrip("var")
             # They like to call 'bool's 'tinyint(1)' and introspection makes that a integer
             # just convert it back to it's proper type, a bool is a bool and nothing else.
-            if db_type=='integer' and description[1]==FIELD_TYPE.TINY and description[4]==1:
+            if db_type == 'integer' and description[1] == FIELD_TYPE.TINY and description[4] == 1:
                 db_type = 'bool'
-            if db_type=='integer' and description[1]==FIELD_TYPE.SHORT:
-                db_type = 'smallint UNSIGNED' # FIXME: what about if it's not UNSIGNED ?
+            if db_type == 'integer' and description[1] == FIELD_TYPE.SHORT:
+                db_type = 'smallint UNSIGNED'  # FIXME: what about if it's not UNSIGNED ?
         return db_type
 
+
 class SqliteSQLDiff(SQLDiff):
     # Unique does not seem to be implied on Sqlite for Primary_key's
     # if this is more generic among databases this might be usefull
     # to add to the superclass's find_unique_missing_in_db method
     def find_unique_missing_in_db(self, meta, table_indexes, table_name):
-        for field in meta.fields:
+        for field in all_local_fields(meta):
             if field.unique:
-                attname = field.attname
+                attname = field.db_column or field.attname
                 if attname in table_indexes and table_indexes[attname]['unique']:
                     continue
-                if table_indexes[attname]['primary_key']:
+                if attname in table_indexes and table_indexes[attname]['primary_key']:
                     continue
                 self.add_difference('unique-missing-in-db', table_name, attname)
 
@@ -443,14 +484,17 @@ class SqliteSQLDiff(SQLDiff):
         if field:
             field_type = self.get_field_model_type(field)
             # Fix char/varchar inconsistencies
-            if self.strip_parameters(field_type)=='char' and self.strip_parameters(db_type)=='varchar':
+            if self.strip_parameters(field_type) == 'char' and self.strip_parameters(db_type) == 'varchar':
                 db_type = db_type.lstrip("var")
         return db_type
 
+
 class PostgresqlSQLDiff(SQLDiff):
     DATA_TYPES_REVERSE_OVERRIDE = {
-        20: 'IntegerField',
         1042: 'CharField',
+        # postgis types (TODO: support is very incomplete)
+        17506: 'django.contrib.gis.db.models.fields.PointField',
+        55902: 'django.contrib.gis.db.models.fields.MultiPolygonField',
     }
 
     # Hopefully in the future we can add constraint checking and other more
@@ -464,8 +508,8 @@ class PostgresqlSQLDiff(SQLDiff):
     ORDER BY CASE WHEN contype='f' THEN 0 ELSE 1 END,contype,nspname,relname,conname;
     """
 
-    SQL_FIELD_TYPE_DIFFER = lambda self, style, qn, args:  "%s %s\n\t%s %s %s %s;" % (style.SQL_KEYWORD('ALTER TABLE'), style.SQL_TABLE(qn(args[0])), style.SQL_KEYWORD('ALTER'), style.SQL_FIELD(qn(args[1])), style.SQL_KEYWORD("TYPE"), style.SQL_COLTYPE(args[2]))
-    SQL_FIELD_PARAMETER_DIFFER = lambda self, style, qn, args:  "%s %s\n\t%s %s %s %s;" % (style.SQL_KEYWORD('ALTER TABLE'), style.SQL_TABLE(qn(args[0])), style.SQL_KEYWORD('ALTER'), style.SQL_FIELD(qn(args[1])), style.SQL_KEYWORD("TYPE"), style.SQL_COLTYPE(args[2]))
+    SQL_FIELD_TYPE_DIFFER = lambda self, style, qn, args: "%s %s\n\t%s %s %s %s;" % (style.SQL_KEYWORD('ALTER TABLE'), style.SQL_TABLE(qn(args[0])), style.SQL_KEYWORD('ALTER'), style.SQL_FIELD(qn(args[1])), style.SQL_KEYWORD("TYPE"), style.SQL_COLTYPE(args[2]))
+    SQL_FIELD_PARAMETER_DIFFER = lambda self, style, qn, args: "%s %s\n\t%s %s %s %s;" % (style.SQL_KEYWORD('ALTER TABLE'), style.SQL_TABLE(qn(args[0])), style.SQL_KEYWORD('ALTER'), style.SQL_FIELD(qn(args[1])), style.SQL_KEYWORD("TYPE"), style.SQL_COLTYPE(args[2]))
 
     def __init__(self, app_models, options):
         SQLDiff.__init__(self, app_models, options)
@@ -483,19 +527,19 @@ class PostgresqlSQLDiff(SQLDiff):
         if not db_type:
             return
         if field:
-            if field.primary_key and db_type=='integer':
+            if field.primary_key and db_type == 'integer':
                 db_type = 'serial'
             if table_name:
                 tablespace = field.db_tablespace
-                if tablespace=="":
+                if tablespace == "":
                     tablespace = "public"
-                check_constraint = self.check_constraints.get((tablespace, table_name, field.attname),{}).get('pg_get_constraintdef', None)
+                check_constraint = self.check_constraints.get((tablespace, table_name, field.attname), {}).get('pg_get_constraintdef', None)
                 if check_constraint:
                     check_constraint = check_constraint.replace("((", "(")
                     check_constraint = check_constraint.replace("))", ")")
                     check_constraint = '("'.join([')' in e and '" '.join(e.split(" ", 1)) or e for e in check_constraint.split("(")])
                     # TODO: might be more then one constraint in definition ?
-                    db_type += ' '+check_constraint
+                    db_type += ' ' + check_constraint
         return db_type
 
     """
@@ -515,6 +559,7 @@ DATABASE_SQLDIFF_CLASSES = {
     'oracle': GenericSQLDiff
 }
 
+
 class Command(BaseCommand):
     option_list = BaseCommand.option_list + (
         make_option('--all-applications', '-a', action='store_true', dest='all_applications',
@@ -526,7 +571,7 @@ class Command(BaseCommand):
         make_option('--output_text', '-t', action='store_false', dest='sql', default=True,
                     help="Outputs the differences as descriptive text instead of SQL"),
     )
-    
+
     help = """Prints the (approximated) difference between models and fields in the database for the given app name(s).
 
 It indicates how columns in the database are different from the sql that would
@@ -538,10 +583,14 @@ to check/debug ur models compared to the real database tables and columns."""
     args = '<appname appname ...>'
 
     def handle(self, *app_labels, **options):
+        from django import VERSION
+        if VERSION[:2] < (1, 0):
+            raise CommandError("SQLDiff only support Django 1.0 or higher!")
+
         from django.db import models
         from django.conf import settings
 
-        if settings.DATABASE_ENGINE =='dummy':
+        if settings.DATABASE_ENGINE == 'dummy':
             # This must be the "dummy" database backend, which means the user
             # hasn't set DATABASE_ENGINE.
             raise CommandError("Django doesn't know which syntax to use for your SQL statements,\n" +
@@ -562,10 +611,16 @@ to check/debug ur models compared to the real database tables and columns."""
             for app in app_list:
                 app_models.extend(models.get_models(app))
 
+        ## remove all models that are not managed by Django
+        #app_models = [model for model in app_models if getattr(model._meta, 'managed', True)]
+
         if not app_models:
             raise CommandError('Unable to execute sqldiff no models founds.')
 
-        cls = DATABASE_SQLDIFF_CLASSES.get(settings.DATABASE_ENGINE, GenericSQLDiff)
+        engine = settings.DATABASE_ENGINE
+        if not engine:
+            engine = connection.__module__.split('.')[-2]
+        cls = DATABASE_SQLDIFF_CLASSES.get(engine, GenericSQLDiff)
         sqldiff_instance = cls(app_models, options)
         sqldiff_instance.find_differences()
         sqldiff_instance.print_diff(self.style)
